
/*
 put all the standard headers in GenCurveFit.h, including those from wavemetrics
 */
#include "XOPStandardHeaders.h"
#include "multiopenfiles.h"
#include <Windows.h>

#define MAX_NUM_FILES 100

// Structure used during the filtering of files.
struct NavFileTypeInfo {
	char titles[256];				// e.g. "data,text"
	char extensions[256];			// e.g., ".txt,.itx"
};

typedef struct NavFileTypeInfo NavFileTypeInfo;
typedef struct NavFileTypeInfo* NavFileTypeInfoPtr;
typedef struct NavFileTypeInfo** NavFileTypeInfoHandle;

/*	NavParseFileFilterString(fileFilterStr, ftiHPtr)
	
	Parses the fileFilterStr into a form more easily used by the file filter function.	
	
	If the fileFilterStr can be parsed correctly, NavParseFileFilterString
	returns a newly created NavFileTypeInfoHandle via ftiHPtr
	and returns a function result of 0. You must dispose of the handle
	when you are finished with it.
	
	If there is an error in parsing, no handle is returned and the function
	result is non-zero.
*/
static int NavParseFileFilterString(const char* fileFilterStr, NavFileTypeInfoHandle* ftiHPtr){
	NavFileTypeInfoHandle ftiH;
	NavFileTypeInfo fti;
	const char* p;
	const char* p1;
	const char* p2;
	const char* p3;
	int runningCount = 0;
	char temp[256];

	*ftiHPtr = NULL;
	
	ftiH = (NavFileTypeInfoHandle)NewHandle(0L);
	
	p = fileFilterStr;
	while(*p != 0) {
		MemClear(&fti, sizeof(fti));
		
		// Now get the extensions associated with this menu item.
				
		p1 = strchr(p, ';');
		if (p1 == NULL) {
			DisposeHandle((Handle)ftiH);
			return 5;						// Error - no semicolon to mark end of extensions.
		}
		
		if (p1 - p >= sizeof(fti.titles)) {
			DisposeHandle((Handle)ftiH);
			return 6;						// Error - extensions string too long.
		}
		strncpy(fti.titles, p, p1-p);

		//here p points to the start of the extensions string, p1 points to the semicolon at the end.
		//we need to get the individual extensions in the list now.
		if (p1 - p >= sizeof(temp)){
			DisposeHandle((Handle)ftiH);
			return 6;
		}
		runningCount=0;
		memset(temp, 0, sizeof(temp));
		strncpy(temp, p, p1-p);
		p2 = temp;
		p3 = temp;
		
		while(*p2){
			p3 = strchr(p2,',');
			if(p3){
				if( (sizeof(fti.extensions) - runningCount > 2+(p3-p2))){
					memcpy(fti.extensions+runningCount,"*",sizeof(char));
					runningCount+=1;
					memcpy(fti.extensions+runningCount, p2, p3-p2);
					runningCount += p3-p2;
					memcpy(fti.extensions+runningCount,";",sizeof(char));
					runningCount+=1;
				} else {
					DisposeHandle((Handle)ftiH);
					return 6;
				}
			} else {
				p3=temp + strlen(temp);
				if((sizeof(fti.extensions) - runningCount > 1+(p3-p2))){
					memcpy(fti.extensions+runningCount,"*",sizeof(char));
					runningCount+=1;
					memcpy(fti.extensions+runningCount, p2, p3-p2);
					runningCount += p3-p2;
				} else {
					DisposeHandle((Handle)ftiH);
					return 6;
				}				
			}
			p2 = p3+1;
		};
		
		p = p1 + 1;							// Skip past extensions to next menu item, if any.
		
		// Add the NavFileTypeInfo record to the handle.
		{
			long origNumBytes = GetHandleSize((Handle)ftiH);
			SetHandleSize((Handle)ftiH, origNumBytes + sizeof(NavFileTypeInfo));
			if (MemError()) {
				DisposeHandle((Handle)ftiH);
				return 7;
			}
			memcpy((char*)*ftiH + origNumBytes, &fti, sizeof(NavFileTypeInfo));
		}
	}
	
	*ftiHPtr = ftiH;
	return 0;
}

#ifdef _WINDOWS_		// [
static UINT APIENTRY	// Hook for open or save file dialogs.
OpenOrSaveFileNameHook(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{	
	HWND hMainDlg;
	
	/*	Because we use the OFN_EXPLORER flag and we specify a hook function, Windows
		creates a child dialog for us and the hdlg parameter to this hook is the child
		dialog.
	*/
	hMainDlg = GetParent(hdlg);
	if (hMainDlg == NULL)
		return 0;

	switch(uiMsg) {
		case WM_INITDIALOG:
			PositionWinDialogWindow(hMainDlg, NULL);
			break;
	}
	return 0;			// Let default dialog box procedure process the message.
}
#endif		

int Executemultiopenfiles(multiopenfilesRuntimeParamsPtr p){
	int err = 0;
	
	NavFileTypeInfoHandle ftiH;
	int numMenuItems;
	char fileFilterStr[256];
	char *lpstrFilter = NULL;
	long totalSz = 0, ii=0;
//	char *fileFilterStr = "All\0*.*\0Text\0*.TXT\0";
	int *fileIndexPtr = NULL;					// NULL or one-based index into type list

	long theCount, len;
	long V_flagRes = -1;
	char windowStr[256];
	char initialDir[MAX_PATH_LEN+1];
	char initialDir2[MAX_PATH_LEN+1];
	char urlOut[MAX_PATH_LEN+1];
	Handle urlOutHand = NULL;
	char *tempP, *tempP1; 
	int hState; 
	int cancelled = 0;
		
	OPENFILENAME ofn;
	char filePath2[MAX_NUM_FILES * (MAX_PATH_LEN+1)];
	memset(initialDir,0,sizeof(initialDir));
	memset(initialDir2,0,sizeof(initialDir2));
	memset(filePath2,0,sizeof(filePath2));
	memset(windowStr,0,sizeof(windowStr));
	memset(fileFilterStr,0,sizeof(fileFilterStr));


	// Flag parameters.
	
	if (p->FFlagEncountered) {
		// Parameter: p->FFlagStrH (test for NULL handle before using)
		if(p->FFlagStrH==NULL){
			err = OH_EXPECTED_STRING;
			goto done;
		}
		if(err = GetCStringFromHandle(p->FFlagStrH, fileFilterStr, 255))
			goto done;
	}

	if (p->PFlagEncountered) {
		// Parameter: p->PFlagName
		if(err = GetPathInfo2(p->PFlagName, initialDir))
			goto done;
	}

	if (strlen(initialDir) == 0) {
		GetStandardFileWinPath(initialDir2);	// Get Igor's open file dialog directory.
	} else {
		strcpy(initialDir2, initialDir);
		SetStandardFileWinPath(initialDir);		// Sets initial directory for next open file dialog. This will be overridden below, but not if the user cancels.
	}

	urlOutHand = NewHandle(0);
	if(urlOutHand == NULL){
		err = NOMEM;
		goto done;
	}
	
	// Parse filter strings, if any. This defines Show menu items.
	ftiH = NULL;
	numMenuItems = 0;
	if (fileFilterStr != 0) {								// Not empty fileFilterStr?
		if(err = NavParseFileFilterString(fileFilterStr, &ftiH)){	// ftiH will be NULL if fileFilterStr is not correctly formed.
			err = COULDNT_PARSE_FILTER;
			goto done;
		}

		if (ftiH != NULL)
			numMenuItems = GetHandleSize((Handle)ftiH) / sizeof(NavFileTypeInfo);

		for(ii=0 ; ii<numMenuItems ; ii+=1){
			totalSz += 2+ (2*strlen((*ftiH)[ii].extensions));
		}
		totalSz += 1;
		lpstrFilter = (char*) malloc(totalSz);
		if(lpstrFilter == NULL){
			err = NOMEM; goto done;
		}
		memset(lpstrFilter, 0, totalSz);
		tempP = lpstrFilter;
		for(ii=0 ; ii<numMenuItems ; ii+=1){
			strncpy(tempP, (*ftiH)[ii].extensions, strlen((*ftiH)[ii].extensions));
			tempP += sizeof(char) + strlen((*ftiH)[ii].extensions);
			strncpy(tempP, (*ftiH)[ii].extensions, strlen((*ftiH)[ii].extensions));
			tempP += sizeof(char) + strlen((*ftiH)[ii].extensions);
		}
	}

	if (p->MFlagEncountered) {
		// Parameter: p->MFlagStrH (test for NULL handle before using)
		if(p->MFlagStrH == NULL){
			err = OH_EXPECTED_STRING;
			goto done;
		}
		if(err = GetCStringFromHandle(p->MFlagStrH, windowStr, 255))
			goto done;
	}

		MemClear(&ofn, sizeof(ofn));
		ofn.lStructSize = sizeof(ofn);
		ofn.hwndOwner = IgorClientHWND();
		ofn.lpstrFile = filePath2;
		ofn.nMaxFile = sizeof(filePath2);
		ofn.lpstrFilter = lpstrFilter;
		ofn.nFilterIndex = fileIndexPtr==NULL ? 1 : *fileIndexPtr;
		ofn.lpstrTitle = windowStr;
		ofn.lpstrFileTitle = NULL;
		ofn.lpstrInitialDir = initialDir2;
		ofn.lpfnHook = OpenOrSaveFileNameHook;		// Needed to set position of the dialog. Otherwise, it is in top/left corner of screen.
		ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
		ofn.Flags |= OFN_EXPLORER;
		ofn.Flags |= OFN_ENABLEHOOK;				// Needed so that hook will be called.
		ofn.Flags |= OFN_HIDEREADONLY;
		ofn.Flags |= OFN_NOCHANGEDIR;
		ofn.Flags |= OFN_ALLOWMULTISELECT;

		if (GetOpenFileName(&ofn) == 0) {
			int err;
			err = CommDlgExtendedError();			// err will be zero if cancel.
			if (err == 0){
				cancelled = 1;
				goto done;
			}
			// We got an error other than cancel.
			*filePath2 = 0;							// HR, 021114: Clear possible bad fields
			*initialDir2 = 0;						// and try again.
			if (GetOpenFileName(&ofn) != 0) {		// Succeeded this time?
				err = 0;
			}
			else {
				if (CommDlgExtendedError() == 0){
					cancelled = 1;
					goto done;						// User canceled.
				}			
				// Report the original error.
				err = WindowsErrorToIgorError(err);
				IgorError("multiopenfiles", err);
				return err;
			}
		}
		
		if (fileIndexPtr != NULL)
			*fileIndexPtr = ofn.nFilterIndex;
		
		//reuse initialDir to put the directory in
		memset(initialDir,0,sizeof(initialDir));
		if(strlen(filePath2)>0)
			strncpy(initialDir, filePath2, sizeof(initialDir));

		//work out if there is more than one file
		//must have multiple files
		if(filePath2[(ofn.nFileOffset)-1] == 0){
			tempP = &filePath2[ofn.nFileOffset];
			tempP1 = strchr(tempP, 0);
		
			while(tempP1 != NULL){
				if(err = PtrAndHand(initialDir, urlOutHand, strlen(initialDir)))
					goto done;
				if(initialDir[strlen(initialDir)-1] != '\\'){
					if(err = PtrAndHand("\\", urlOutHand, 1))
						goto done;
				}
				if(err = PtrAndHand(tempP, urlOutHand, strlen(tempP)))
					goto done;
				if(err = PtrAndHand(";", urlOutHand, 1))
					goto done;
				tempP = tempP1 + sizeof(char);
				V_flagRes = 0;
				if(*tempP == 0)
					break;
				tempP1 = strchr(tempP, 0);
			}
		} else {	//must have single file
			if(err = PtrAndHand(filePath2, urlOutHand, strlen(filePath2)))
				goto done;
			V_flagRes = 0;
		}

	SetStandardFileWinPath(initialDir);			// Update Igor's open file dialog directory.

	//////////CLEANUP///////////////	
done:
	if(err || cancelled){
		V_flagRes=-1;
		SetOperationStrVar("S_filename", ""); 
	}

	SetOperationNumVar("V_flag", V_flagRes);

	if (GetHandleSize(urlOutHand) > 0 && V_flagRes==0) { 
		err = SetOperationStrVar2("S_filename", *urlOutHand, GetHandleSize(urlOutHand));

	if (ftiH != NULL)
		DisposeHandle((Handle)ftiH);
	if(urlOutHand)
		DisposeHandle(urlOutHand);
	if(lpstrFilter)
		free(lpstrFilter);
	
	return err;
}